@@ -152,6 +152,7 @@ class OpenADRClient:
         if self.report_queue_task:
             self.report_queue_task.cancel()
         await self.client_session.close()
+        await asyncio.sleep(0)
 
     def add_handler(self, handler, callback):
         """
@@ -706,10 +707,11 @@ class OpenADRClient:
         except Exception as err:
             logger.error(f"The incoming message could not be parsed or validated: {err}")
             return None, {}
-        if message_payload['response']['response_code'] != 200:
-            logger.warning("We got a non-OK OpenADR response from the server: "
-                           f"{message_payload['response']['response_code']}: "
-                           f"{message_payload['response']['response_description']}")
+        if 'response' in message_payload and 'response_code' in message_payload['response']:
+            if message_payload['response']['response_code'] != 200:
+                logger.warning("We got a non-OK OpenADR response from the server: "
+                               f"{message_payload['response']['response_code']}: "
+                               f"{message_payload['response']['response_description']}")
         return message_type, message_payload
 
     async def _on_event(self, message):
